/*
 * dumpbb.c - Dump netbb format
 *
 * Written 2002 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dwarfx.h"

#include "util.h"
#include "stmt.h"
#include "io.h"
#include "cpp.h"


extern void open_elf(const char *name);
extern const void *mmap_file(const char *name,int *size);
extern int yyparse(void);
extern struct stmt *script;
extern const char *default_script;

int debug = 0;


/* ----- Argument processing ----------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-c script_file] [-d] [cpp_option ...] "
      "[elf_file ... blackbox_file]\n",name);
    fprintf(stderr,"  -c  use specified processing script (- for stdin)\n");
    fprintf(stderr,"  -d  enable debugging output\n");
    fprintf(stderr,"  cpp_option            -Idir, -Dname[=value], or "
      "-Uname\n");
    exit(1);
}


int main(int argc,char **argv)
{
    const char *config_script = NULL;
    int stdin = 0,c;

    while ((c = getopt(argc,argv,"c:dD:U:I:")) != EOF)
	switch (c) {
	    static char opt[3] = "-?";

	    case 'c':
		config_script = optarg;
		break;
	    case 'd':
		debug = 1;
		break;
	    case 'D':
	    case 'U':
	    case 'I':
		opt[1] = c;
		add_cpp_arg(stralloc(opt));
		add_cpp_arg(optarg);
		break;
	    default:
		usage(*argv);
	}
    if (!config_script && argc < optind+2) {
	fprintf(stderr,"default script requires an ELF file and a data file\n");
	exit(1);
    }
    if (argc != optind) {
	while (optind < argc-1) {
	    open_elf(argv[optind++]);
	    while (1) {
		const void *p;

		p = dwarf_comp_unit();
		if (!p) break;
		dwx_read_types(p);
	    }
	}
	if (strcmp(argv[optind],"-")) data_open(argv[optind]);
	else {
	    data_open(NULL);
	    stdin = 1;
	}
    }
    init_types();
    if (!config_script) run_cpp_on_string(default_script);
    else {
	if (!strcmp(config_script,"-")) {
	    if (stdin) {
		fprintf(stderr,"can't read stdin more than once\n");
		exit(1);
	    }
	    config_script = NULL;
	}
	run_cpp_on_file(config_script);
    }
    (void) yyparse();
    execute(script);
    stmt_end(NULL);
    return 0; /* not reached */

#if 0
    const struct dwx_type *type;
    const void *data,*end;
    int size;

    data = mmap_file(argv[optind+1],&size);
    end = data+size;
    type = dwx_lookup_type(dwx_struct,"sock",-1,0);
    if (!type) {
	fprintf(stderr,"struct sock not found\n");
	exit(1);
    }
    dwx_output_data(stdout,type,data);
    data += dwx_sizeof(type);
    type = dwx_lookup_type(dwx_struct,"sk_buff",-1,0);
    if (!type) {
	fprintf(stderr,"struct sk_buff not found\n");
	exit(1);
    }
    while (data != end) {
	unsigned long head,tail;

	if (data > end) {
	    fprintf(stderr,"dump file size error\n");
	    exit(1);
	}
	dwx_output_data(stdout,type,data);
	head = dwx_lookup_unsigned(type,data,"head");
	tail = dwx_lookup_unsigned(type,data,"tail");
	data += dwx_sizeof(type);
	dwx_output_hexdump(stdout,data,tail-head,0);
	data += tail-head;
    }
    return 0;
#endif
}
